/*(c) 2006 Chritian B. Huebschle*/
struct Vector3 {
  GLfloat x, y, z;
  inline Vector3( void ){}
  inline Vector3( const GLfloat& _x, const GLfloat& _y, const GLfloat& _z ) : 
	  x(_x), y(_y), z(_z) {
	    ;
	  }
  inline Vector3& operator *= ( const GLfloat& d ){
    x *= d;
    y *= d;
    z *= d;
    return *this;
  }
  inline Vector3& operator += ( const Vector3& v ){
    x += v.x;
    y += v.y;
    z += v.z;
    return *this;
  } 
  inline Vector3& operator += ( const GLfloat& v ){
    x += v;
    y += v;
    z += v;
    return *this;
  }
};
inline bool operator == (const Vector3& v1, const Vector3& v2 ) {
  return ((v1.x==v2.x)&&(v1.y==v2.y)&&(v1.z==v2.z));
}
inline Vector3 operator + ( const Vector3& v1, const Vector3& v2 ) {
  Vector3 t;
  t.x = v1.x + v2.x;
  t.y = v1.y + v2.y;
  t.z = v1.z + v2.z;
  return t;
}
inline Vector3 operator - ( const Vector3& v1, const Vector3& v2 ) {
  Vector3 t;
  t.x = v1.x - v2.x;
  t.y = v1.y - v2.y;
  t.z = v1.z - v2.z;
  return t;
}
inline Vector3 operator * ( const Vector3& v, const GLfloat& d ) {
  Vector3 t;
  t.x = v.x*d;
  t.y = v.y*d;
  t.z = v.z*d;
  return t;
}
inline Vector3 operator * ( const GLfloat& d, const Vector3& v ) {
  Vector3 t;
  t.x = v.x*d;
  t.y = v.y*d;
  t.z = v.z*d;
  return t;
}
inline Vector3 operator % ( const Vector3& v1, const Vector3& v2 ) {
  Vector3 t;
  t.x = v1.y*v2.z - v2.y*v1.z;
  t.y = v1.z*v2.x - v2.z*v1.x;
  t.z = v1.x*v2.y - v2.x*v1.y;
  return t;
}
inline GLfloat operator * ( const Vector3& v1, const Vector3& v2 ) {
  return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z;
}
inline GLfloat Norm( const Vector3& v ) {
  return v.x*v.x + v.y*v.y + v.z*v.z;
}
inline GLfloat Distance( const Vector3& v1, const Vector3& v2 ) {
  return Norm(v1 - v2);
}
inline Vector3& Normalize( Vector3 v ) {
  if (Norm(v))  return v *= 1.0/sqrt(Norm(v));
  else return v*=1.0;
}
struct Polygn {
  Vector3 vertex;
  int n;
  int ii[13];
};
struct ort {
  Vector3 vertex;
  Vector3 normal;
  GLfloat color;
  char direct;
};
struct Node {
  int index;
  unsigned char flag; 
  inline operator unsigned char (){return flag;}
};
extern Polygn *PGN;
extern ort *Orte;
///////////////////////////////////////////////////////////////////////////////
class Isosurf {
  FILE * f;  
  int pcn_start;
  GLfloat  *data; 
  GLfloat *imdata;  
  GLfloat iso_level;      
  //GLfloat grad_cutoff;
  int VCN;
  int size;
  int size2;
  int breit,hoch,tief,bh,bt;
  float untersumme,obersumme,Volumen,Volumen2,deV;
  Vector3 *grad;
  Node *nodex;   
  Node *nodey;   
  Node *nodez;     
  int plana;
  GLfloat max;
  GLfloat min;
  GLfloat dL,ddim;
	public:   
  Isosurf( void ) {grad=NULL;nodex=NULL;nodey=NULL;nodez=NULL;erstma=1;plana=0;} 
  void Assign( GLfloat *_data, GLfloat *_mdata, GLfloat _iso_level ,int br, int ho, int ti,GLfloat dim){
    //int mem= br * ho * ti * 3 * sizeof(Node)+br * ho * ti *sizeof(Vector3);
    if (dim<0) {plana=1;dim*=-1.0;}else plana=0;

    pcn_start=PCN;
    breit=br;
    hoch=ho;
    tief=ti;
    bh=br*ho;
    erstma=1;
    bt=br*ti;
    //fprintf(LANG,"Allocate %dB = %5.2fkB = %5.2fMB = %5.2fGB Memory!\n",mem,mem/1024.0,mem/(1024.0*1024.0),mem/(1024.0*1024.0*1024.0));
    untersumme=0.0f;
    obersumme=0.0f;
    Volumen2=0.0f;
    deV=(double)(dim*dim*dim)/(double)(br*ho*ti);
    ddim=dim;
    max=-1e38;
    min=1e38;
    dL=1.0/br;
    size=br;
    size2=br*ho;

    if (( grad =(Vector3*)malloc(sizeof(Vector3)*br*ho*ti))==NULL) {
      fprintf(stderr ,"Less Memory(X)!!\n");
      exit(1);  
    } 

    if (( nodex =(Node*)malloc(sizeof(Node)*br*ho*ti*2))==NULL) {
      fprintf(stderr ,"Less Memory(X)!!\n");
      exit(1);  
    } 

    if (( nodey =(Node*)malloc(sizeof(Node)*br*ho*ti*2))==NULL) { 
      fprintf(stderr ,"Less Memory(Y)!!\n");
      exit(1); 
    }
    if (( nodez =(Node*)malloc(sizeof(Node)*br*ho*ti*2))==NULL) {
      fprintf(stderr ,"Less Memory(Z)!!\n");
      exit(1); 
    }

    data=_data;
    //    printf("TEST: %p %p\n",(void*)_mdata,(void*)_data);
    if (_mdata==NULL) imdata=data;
    else imdata=_mdata;

    //printf("TEST: %p %p\n",(void*)imdata,(void*)data);
    iso_level = _iso_level;
    //grad_cutoff=_grad_cutoff;

  }
  void Free(void);
  void Legende( void );
  void setMinMax(GLfloat Min,GLfloat Max){
    min=Min;
    max=Max;
  }
  void makeGrad(void){
    simpelGrad();
  };
  static  int Xcmp(const void *p1,const void *p2){
    Polygn *P1 =(Polygn*) p1;
    Polygn *P2 =(Polygn*) p2;
    float cmp = P1->vertex.x-P2->vertex.x;
    if (cmp>0.0f) return 1; else 
      if (cmp<0.0f) return -1; else return 0;
  }
  static  int xcmp(const void *p1,const void *p2){
    Polygn *P1 =(Polygn*) p1;
    Polygn *P2 =(Polygn*) p2;
    float cmp = P1->vertex.x-P2->vertex.x;
    if (cmp>0.0f) return -1; else
      if (cmp<0.0f) return 1; else return 0;
  }
  static  int Ycmp(const void *p1,const void *p2){
    Polygn *P1 =(Polygn*) p1;
    Polygn *P2 =(Polygn*) p2;
    float cmp = P1->vertex.y-P2->vertex.y;
    if (cmp>0.0f) return 1; else
      if (cmp<0.0f) return -1; else return 0;
  }
  static  int ycmp(const void *p1,const void *p2){
    Polygn *P1 =(Polygn*) p1;
    Polygn *P2 =(Polygn*) p2;
    float cmp = P1->vertex.y-P2->vertex.y;
    if (cmp>0.0f) return -1; else
      if (cmp<0.0f) return 1; else return 0;
  }
  static  int Zcmp(const void *p1,const void *p2){
    Polygn *P1 =(Polygn*) p1;
    Polygn *P2 =(Polygn*) p2;
    float cmp = P1->vertex.z-P2->vertex.z;
    if (cmp>0.0f) return 1; else
      if (cmp<0.0f) return -1; else return 0;
  }
  static  int zcmp(const void *p1,const void *p2){
    Polygn *P1 =(Polygn*) p1;
    Polygn *P2 =(Polygn*) p2;
    float cmp = P1->vertex.z-P2->vertex.z;
    if (cmp>0.0f) return -1; else
      if (cmp<0.0f) return 1; else return 0;
  }
  void PXsort( void) {
    qsort((void *) PGN,(size_t) PCN,sizeof( Polygn),Xcmp);
  }
  void Pxsort( void) {
    qsort((void *) PGN,(size_t) PCN,sizeof( Polygn),xcmp);
  }
  void PYsort( void) {
    qsort((void *) PGN,(size_t) PCN,sizeof( Polygn),Ycmp);
  }
  void Pysort( void) {
    qsort((void *) PGN,(size_t) PCN,sizeof( Polygn),ycmp);
  }
  void PZsort( void) {
    qsort((void *) PGN,(size_t) PCN,sizeof( Polygn),Zcmp);
  }
  void Pzsort( void) {
    qsort((void *) PGN,(size_t) PCN,sizeof( Polygn),zcmp);
  }
  void Make1( void ){  
    //	printf("beginne mit polygon %d und Ozi %d\n",PCN,ozi); 
    pcn_start=PCN;//printf("OOOO\n");    
    CalcVertexes();
    //printf("A\n");    
    for (int i=0;i<regula;i++) Regularize();
    if(!regula) CalcNormals();

    //printf("B\n");
    minmax();
    //printf("C\n");fflush(stdout);
    VCN=0;//hier mussen fruhere beachtet werden
    //PCN=0;
    //printf("nullen %p ",PGN);
    //PGN=NULL;	
    //printf("genullt %p \n",PGN);
    if ((!plana)&&(strcmp(surf2,""))&&(f=fopen(surf2,"w"))){
      fprintf(f,"%d\n",ozi);
      for (int i=0;i<ozi;i++)
	fprintf(f,"%-6d  %9.6f %9.6f %9.6f   %9.6f %9.6f %9.6f  %12.7f\n",i,
			Orte[i].vertex.x,
			Orte[i].vertex.y,
			Orte[i].vertex.z,
			Orte[i].normal.x,
			Orte[i].normal.y,
			Orte[i].normal.z,
			Orte[i].color);}
    else f=NULL;
    //
    MakeElements(); 
    //if (f!=NULL) fprintf(f,"!!!!!!!!!!!!!!!!!!!!!\n");
    printf("OZI %d Verts%d Polys %d\n",ozi,VCN,PCN);
    //OZI 10758 Verts28626 Polys 7508 ==7 iterat
    //OZI 10758 Verts29806 Polys 7754 ==1 iterat
    //OZI 10758 Verts34378 Polys 9034 ==0 iterat
    if (f!=NULL) fclose(f);
  }
  void Make2( void ){
   // if (strcmp(surf2,"")) f=fopen(surf2,"a");
    MakeElements2();
    //if (f!=NULL) fprintf(f,"!!!!!!!!!!!!!!!!!!!!!\n");
  }
  void Make3( void ){
    MakeElements3();
    //if (f!=NULL) fprintf(f,"!!!!!!!!!!!!!!!!!!!!!\n");
  }
  void Make4( void ){
    MakeElements4();
    //if (f!=NULL) fprintf(f,"!!!!!!!!!!!!!!!!!!!!!\n");
  }
  void Make5( void ){
    MakeElements5();
    //if (f!=NULL) fprintf(f,"!!!!!!!!!!!!!!!!!!!!!\n");
  }
  void Make6( void ){
    MakeElements6();
  //  if (f!=NULL) fprintf(f,"!!!!!!!!!!!!!!!!!!!!!\n");

  //  if (f!=NULL) fclose(f);
  }

  void DrawPlys();
  void Regularize(void);
	private:  
  int erstma;
  inline int Intersect( GLfloat& vm, GLfloat& vp ){
    return vm*vp <= 0.0 && (vm<0.0 || vp<0.0);
  }
  void CalcVertex( int ix, int iy, int iz );
  void CalcVertexes( void );
  int IndexSelected( Node& node0, Node& node1, Node& node2, Node& node3 );
  Vector3& VectorSelected( Node& node0, Node& node1, Node& node2, Node& node3 );
  GLfloat CalcNormalX( int ix, int iy, int iz );
  GLfloat CalcNormalY( int ix, int iy, int iz );
  GLfloat CalcNormalZ( int ix, int iy, int iz );
  //  Vector3 cN( int ix, int iy, int iz);
  void CalcNormals( void );
  void makeFaces(int nn, Node poly[] );
  // void DrawPolygon( int n, Node polygon[] );
  void MakeElement( int ix, int iy, int iz,int s1,int s2 );
  void MakeElements( void );  
  void MakeElements2( void ); 
  void MakeElements3( void ); 
  void MakeElements4( void ); 
  void MakeElements5( void ); 
  void MakeElements6( void );  
  void Farbverlauf (GLfloat wrt);
  void minmax(void);  
  void simpelGrad(void);
  void save(void);
};
///////////////////////////////////////////////////////////////////////////////

void Isosurf::simpelGrad(void){
  GLfloat hier;
  for (int i=1; i<breit-1;i++){
    for (int j=1; j<hoch-1;j++){
      for (int k=1; k<tief-1;k++){
	hier=data[(i  )+(j  )*size+(k  )*size2];
	grad[i+j*size+k*size2].x=(
			-1*(data[(i-1)+(j-1)*size+(k-1)*size2]-hier)+//0  
			(data[(i+1)+(j-1)*size+(k-1)*size2]-hier)+//2
			-1*(data[(i-1)+(j  )*size+(k-1)*size2]-hier)+//3
			(data[(i+1)+(j  )*size+(k-1)*size2]-hier)+//5  
			-1*(data[(i-1)+(j+1)*size+(k-1)*size2]-hier)+//6  
			(data[(i+1)+(j+1)*size+(k-1)*size2]-hier)+//8  
			-1*(data[(i-1)+(j-1)*size+(k-1)*size2]-hier)+//9 
			(data[(i+1)+(j-1)*size+(k  )*size2]-hier)+//11 
			-1*(data[(i-1)+(j  )*size+(k  )*size2]-hier)+//12 	    					
			(data[(i+1)+(j  )*size+(k  )*size2]-hier)+//14 
			-1*(data[(i-1)+(j+1)*size+(k  )*size2]-hier)+//15
			(data[(i+1)+(j+1)*size+(k  )*size2]-hier)+//17
			-1*(data[(i-1)+(j-1)*size+(k+1)*size2]-hier)+//18 
			(data[(i+1)+(j-1)*size+(k+1)*size2]-hier)+//20 
			-1*(data[(i-1)+(j  )*size+(k+1)*size2]-hier)+//21
			(data[(i+1)+(j  )*size+(k+1)*size2]-hier)+//23 
			-1*(data[(i-1)+(j+1)*size+(k+1)*size2]-hier)+//24 
			(data[(i+1)+(j+1)*size+(k+1)*size2]-hier))/26.0;//26 

	grad[i+j*size+k*size2].y=(
			-1*(data[(i-1)+(j-1)*size+(k-1)*size2]-hier)+//0
			-1*(data[(i  )+(j-1)*size+(k-1)*size2]-hier)+//1
			-1*(data[(i+1)+(j-1)*size+(k-1)*size2]-hier)+//2  
			(data[(i-1)+(j+1)*size+(k-1)*size2]-hier)+//6  
			(data[(i  )+(j+1)*size+(k-1)*size2]-hier)+//7  
			(data[(i+1)+(j+1)*size+(k-1)*size2]-hier)+//8  
			-1*(data[(i-1)+(j-1)*size+(k-1)*size2]-hier)+//9  
			-1*(data[(i  )+(j-1)*size+(k  )*size2]-hier)+//10 
			-1*(data[(i+1)+(j-1)*size+(k  )*size2]-hier)+//11
			(data[(i-1)+(j+1)*size+(k  )*size2]-hier)+//15 
			(data[(i  )+(j+1)*size+(k  )*size2]-hier)+//16 
			(data[(i+1)+(j+1)*size+(k  )*size2]-hier)+//17
			-1*(data[(i-1)+(j-1)*size+(k+1)*size2]-hier)+//18 
			-1*(data[(i  )+(j-1)*size+(k+1)*size2]-hier)+//19 
			-1*(data[(i+1)+(j-1)*size+(k+1)*size2]-hier)+//20 
			(data[(i-1)+(j+1)*size+(k+1)*size2]-hier)+//24 
			(data[(i  )+(j+1)*size+(k+1)*size2]-hier)+//25 
			(data[(i+1)+(j+1)*size+(k+1)*size2]-hier))/26.0;//26 

	grad[i+j*size+k*size2].z=(
			-1*(data[(i-1)+(j-1)*size+(k-1)*size2]-hier)+//0
			-1*(data[(i  )+(j-1)*size+(k-1)*size2]-hier)+//1
			-1*(data[(i+1)+(j-1)*size+(k-1)*size2]-hier)+//2
			-1*(data[(i-1)+(j  )*size+(k-1)*size2]-hier)+//3  
			-1*(data[(i  )+(j  )*size+(k-1)*size2]-hier)+//4
			-1*(data[(i+1)+(j  )*size+(k-1)*size2]-hier)+//5  
			-1*(data[(i-1)+(j+1)*size+(k-1)*size2]-hier)+//6  
			-1*(data[(i  )+(j+1)*size+(k-1)*size2]-hier)+//7  
			-1*(data[(i+1)+(j+1)*size+(k-1)*size2]-hier)+//8
			(data[(i-1)+(j-1)*size+(k+1)*size2]-hier)+//18 
			(data[(i  )+(j-1)*size+(k+1)*size2]-hier)+//19 
			(data[(i+1)+(j-1)*size+(k+1)*size2]-hier)+//20 
			(data[(i-1)+(j  )*size+(k+1)*size2]-hier)+//21 
			(data[(i  )+(j  )*size+(k+1)*size2]-hier)+//22
			(data[(i+1)+(j  )*size+(k+1)*size2]-hier)+//23 
			(data[(i-1)+(j+1)*size+(k+1)*size2]-hier)+//24 
			(data[(i  )+(j+1)*size+(k+1)*size2]-hier)+//25 
			(data[(i+1)+(j+1)*size+(k+1)*size2]-hier))/26.0;//26 
	if (i==1) {
	  grad[0+j*size+k*size2].x= grad[i+j*size+k*size2].x;
	  grad[0+j*size+k*size2].y= grad[i+j*size+k*size2].y;
	  grad[0+j*size+k*size2].z= grad[i+j*size+k*size2].z;
	}if (j==1) {
	  grad[i+0*size+k*size2].x= grad[i+j*size+k*size2].x;
	  grad[i+0*size+k*size2].y= grad[i+j*size+k*size2].y;
	  grad[i+0*size+k*size2].z= grad[i+j*size+k*size2].z;
	}if (k==1) {
	  grad[i+j*size+0*size2].x= grad[i+j*size+k*size2].x;
	  grad[i+j*size+0*size2].y= grad[i+j*size+k*size2].y;
	  grad[i+j*size+0*size2].z= grad[i+j*size+k*size2].z;
	}
	if (i==breit-2) {
	  grad[breit-1+j*size+k*size2].x= grad[i+j*size+k*size2].x;
	  grad[breit-1+j*size+k*size2].y= grad[i+j*size+k*size2].y;
	  grad[breit-1+j*size+k*size2].z= grad[i+j*size+k*size2].z;
	}if (j==hoch-2) {
	  grad[i+(hoch-1)*size+k*size2].x= grad[i+j*size+k*size2].x;
	  grad[i+(hoch-1)*size+k*size2].y= grad[i+j*size+k*size2].y;
	  grad[i+(hoch-1)*size+k*size2].z= grad[i+j*size+k*size2].z;
	}if (k==tief-2) {
	  grad[i+j*size+(tief-1)*size2].x= grad[i+j*size+k*size2].x;
	  grad[i+j*size+(tief-1)*size2].y= grad[i+j*size+k*size2].y;
	  grad[i+j*size+(tief-1)*size2].z= grad[i+j*size+k*size2].z;
	}
      }
    }
  }
}

void Isosurf::CalcVertex( int ix, int iy, int iz ) {
  GLfloat vo, vx, vy, vz,sig;

  sig = (iso_level>0)?-1:1;
  vo = data[ix+iy*size+iz*size2]  - iso_level;
  if( ix != breit-1 ) vx = data[(1+ix)+iy*size+iz*size2] - iso_level;
  if( iy != hoch-1 ) vy = data[ix+(iy+1)*size+iz*size2] - iso_level;
  if( iz != tief-1 ) vz = data[ix+iy*size+(iz+1)*size2] - iso_level;

  //printf("%p orte %d \n",Orte,ozi);
  if( ix != breit-1 && Intersect(vo,vx) ){
    if ((Orte =(ort*)realloc(Orte,sizeof(ort)*(ozi+1)))==NULL) {
      fprintf(stderr ,"Less Memory(ort) %d!!\n",ozi);
      exit(1);  
    }
    Orte[ozi].vertex= Vector3( ((ix+vo/(vo-vx))+0.5)/breit, (iy+0.5)/hoch, (iz+0.5)/tief );
    Orte[ozi].normal= Normalize(sig*( grad[ix+iy*size+iz*size2]*(1.0-vo/(vo-vx)) +grad[(1+ix)+iy*size+iz*size2]*(1.0-vo/(vo-vx))));
    Orte[ozi].color= imdata[ix+iy*size+iz*size2]*(1.0-vo/(vo-vx))+vo/(vo-vx)*imdata[(1+ix)+iy*size+iz*size2];
    nodex[ix+iy*size+iz*size2].index=ozi;
    nodex[ix+iy*size+iz*size2].flag=1; 
    ozi++;

  }else{
    nodex[ix+iy*size+iz*size2].flag   = 0;
  }
  if( iy != hoch-1 && Intersect(vo,vy) ){
    if ((Orte =(ort*)realloc(Orte,sizeof(ort)*(ozi+1)))==NULL) {
      fprintf(stderr ,"Less Memory(ort) %d!!\n",ozi);
      exit(1);  
    }
    Orte[ozi].vertex=Vector3( (ix+0.5)/breit, (iy+vo/(vo-vy)+0.5)/hoch, (iz+0.5)/tief );
    Orte[ozi].normal=Normalize(sig*( grad[ix+iy*size+iz*size2]*(1.0-vo/(vo-vy)) +grad[ix+(iy+1)*size+iz*size2]*(1.0-vo/(vo-vy))));
    Orte[ozi].color=imdata[ix+iy*size+iz*size2]*(1.0-vo/(vo-vy))+vo/(vo-vy)*imdata[ix+(iy+1)*size+iz*size2];
    nodey[ix+iy*size+iz*size2].index=ozi;
    nodey[ix+iy*size+iz*size2].flag=1;
    ozi++;
  }else{
    nodey[ix+iy*size+iz*size2].flag=0;
  }
  if( iz != tief-1 && Intersect(vo,vz) ){
    if ((Orte =(ort*)realloc(Orte,sizeof(ort)*(ozi+1)))==NULL) {
      fprintf(stderr ,"Less Memory(ort) %d!!\n",ozi);
      exit(1);  
    }
    Orte[ozi].vertex = Vector3( (ix+0.5)/breit, (iy+0.5)/hoch, (iz+vo/(vo-vz)+0.5)/tief );
    Orte[ozi].normal = Normalize(sig*( grad[ix+iy*size+iz*size2]*(1.0-vo/(vo-vz)) +grad[ix+iy*size+(iz+1)*size2]*(1.0-vo/(vo-vz))));
    Orte[ozi].color = imdata[ix+iy*size+iz*size2]*(1.0-vo/(vo-vz))+vo/(vo-vz)*imdata[ix+iy*size+(iz+1)*size2];
    nodez[ix+iy*size+iz*size2].index = ozi;
    nodez[ix+iy*size+iz*size2].flag=1;
    ozi++;
  }else{
    nodez[ix+iy*size+iz*size2].flag  = 0;
  }
}
void Isosurf::CalcVertexes( void ) {
  float mul,fehler,E_us=0.0,E_os=0.0,Elektronen=0.0,ef=0.0;
  for( int ix=0; ix<breit; ix++ ){
    for( int iy=0; iy<hoch; iy++ ){
      for( int iz=0; iz<tief; iz++ ){
	// printf("%4d %4d %4d %7d %7d %p\n",ix,iy,iz,ozi,PCN,Orte);
	CalcVertex(ix,iy,iz);
	if(!plana){
	  if ((ix<size-1)&&(iy<size-1)&&(iz<size-1)){
	    if ((data[(ix)   + (iy*size)   + (iz*size2) ]   > iso_level)&&
			    (data[(ix)   + (iy*size)   + (iz*size2+1) ] > iso_level)&&
			    (data[(ix)   + (iy*size+1) + (iz*size2) ]   > iso_level)&&
			    (data[(ix)   + (iy*size+1) + (iz*size2+1) ] > iso_level)&&
			    (data[(ix+1) + (iy*size)   + (iz*size2) ]   > iso_level)&&
			    (data[(ix+1) + (iy*size)   + (iz*size2+1) ] > iso_level)&&
			    (data[(ix+1) + (iy*size+1) + (iz*size2) ]   > iso_level)&&
			    (data[(ix+1) + (iy*size+1) + (iz*size2+1) ] > iso_level)){
	      untersumme+=deV;	    
	      E_us+=((imdata[(ix)   + (iy*size)   + (iz*size2) ]   )+
			      (imdata[(ix)   + (iy*size)   + (iz*size2+1) ] )+
			      (imdata[(ix)   + (iy*size+1) + (iz*size2) ]   )+
			      (imdata[(ix)   + (iy*size+1) + (iz*size2+1) ] )+
			      (imdata[(ix+1) + (iy*size)   + (iz*size2) ]   )+
			      (imdata[(ix+1) + (iy*size)   + (iz*size2+1) ] )+
			      (imdata[(ix+1) + (iy*size+1) + (iz*size2) ]   )+
			      (imdata[(ix+1) + (iy*size+1) + (iz*size2+1) ] ))/8.0*deV;
	    }

	    if ((data[(ix)   + (iy*size)   + (iz*size2) ]   > iso_level)||
			    (data[(ix)   + (iy*size)   + (iz*size2+1) ] > iso_level)||
			    (data[(ix)   + (iy*size+1) + (iz*size2) ]   > iso_level)||
			    (data[(ix)   + (iy*size+1) + (iz*size2+1) ] > iso_level)||
			    (data[(ix+1) + (iy*size)   + (iz*size2) ]   > iso_level)||
			    (data[(ix+1) + (iy*size)   + (iz*size2+1) ] > iso_level)||
			    (data[(ix+1) + (iy*size+1) + (iz*size2) ]   > iso_level)||
			    (data[(ix+1) + (iy*size+1) + (iz*size2+1) ] > iso_level)){
	      obersumme+=deV;
	      E_os+=((imdata[(ix)   + (iy*size)   + (iz*size2) ]   )+
			      (imdata[(ix)   + (iy*size)   + (iz*size2+1) ] )+
			      (imdata[(ix)   + (iy*size+1) + (iz*size2) ]   )+
			      (imdata[(ix)   + (iy*size+1) + (iz*size2+1) ] )+
			      (imdata[(ix+1) + (iy*size)   + (iz*size2) ]   )+
			      (imdata[(ix+1) + (iy*size)   + (iz*size2+1) ] )+
			      (imdata[(ix+1) + (iy*size+1) + (iz*size2) ]   )+
			      (imdata[(ix+1) + (iy*size+1) + (iz*size2+1) ] ))/8.0*deV;	
	      mul=((data[(ix)   + (iy*size)   + (iz*size2) ]   > iso_level)?0.125:0)+
		      ((data[(ix)   + (iy*size)   + (iz*size2+1) ] > iso_level)?0.125:0)+
		      ((data[(ix)   + (iy*size+1) + (iz*size2) ]   > iso_level)?0.125:0)+
		      ((data[(ix)   + (iy*size+1) + (iz*size2+1) ] > iso_level)?0.125:0)+
		      ((data[(ix+1) + (iy*size)   + (iz*size2) ]   > iso_level)?0.125:0)+
		      ((data[(ix+1) + (iy*size)   + (iz*size2+1) ] > iso_level)?0.125:0)+
		      ((data[(ix+1) + (iy*size+1) + (iz*size2) ]   > iso_level)?0.125:0)+
		      ((data[(ix+1) + (iy*size+1) + (iz*size2+1) ] > iso_level)?0.125:0);
	      Volumen2+=deV*mul;
	    }
	  }
	}
      }
    }
  }
  //fprintf(LANG,"OZI: %d Punkte Ort ist %d(%d) gross Node(%d)\n",ozi,sizeof(ort),ozi*sizeof(ort),sizeof(Node));
  if(!plana){
    Elektronen=(E_os+E_us)/2.0;
    ef=(E_os-E_us)/2.0;
    Volumen=(obersumme+untersumme)/2.0;
    fehler=(obersumme-untersumme)/4.0;   
    printf("\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\nIntegration of the surface volume[A^3]:\n %f < %5.0f(%.0f)  (??%f) < %f\n",untersumme,Volumen,fehler,Volumen2,obersumme);
    printf("If the map grid is 'xd_rho.grd' this is the electron poulation \ninside the isofurface: %f+-%f (Min=%f Max=%f)\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",Elektronen,ef,E_us,E_os);
  }
}
int Isosurf::IndexSelected( Node& node0, Node& node1, Node& node2, Node& node3 ) {
  if( node1 && node2 && node3 ){
    GLfloat d1 = Distance( Orte[node0.index].vertex, Orte[node1.index].vertex ) + 
	    Distance( Orte[node3.index].vertex, Orte[node2.index].vertex );
    GLfloat d2 = Distance( Orte[node0.index].vertex, Orte[node2.index].vertex ) + 
	    Distance( Orte[node3.index].vertex, Orte[node1.index].vertex );
    if( d1 > d2 ) return 2; else return 1;
  }else{
    if(      node1 )   return 1;
    else if( node2 )   return 2;
    else if( node3 )   return 3;
  }
  return 0;
}
Vector3& Isosurf::VectorSelected( Node& node0, Node& node1, Node& node2, Node& node3 ) {
  if( node1 && node2 && node3 ){
    GLfloat d1 = Distance( Orte[node0.index].vertex, Orte[node1.index].vertex ) + 
	    Distance( Orte[node3.index].vertex, Orte[node2.index].vertex );
    GLfloat d2 = Distance( Orte[node0.index].vertex, Orte[node2.index].vertex ) + 
	    Distance( Orte[node3.index].vertex, Orte[node1.index].vertex );
    if( d1 > d2 ) return Orte[node2.index].vertex; else return Orte[node1.index].vertex;
  }else{
    if(      node1 )   return Orte[node1.index].vertex;
    else if( node2 )   return Orte[node2.index].vertex;
    else if( node3 )   return Orte[node3.index].vertex;
  }
  return Orte[node0.index].vertex;
}
GLfloat Isosurf::CalcNormalX( int ix, int iy, int iz ) {
  Vector3 tang[4];

  if(( iy == hoch-1 )||(ix == breit-1)){
    tang[0] = Orte[nodex[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[0] = VectorSelected(nodex[ix+iy*size+iz*size2],nodey[ix+iy*size+iz*size2],
		    nodey[1+ix+iy*size+iz*size2],nodex[ix+(iy+1)*size+iz*size2]);
  }
  if(( iy == 0 )||(ix == breit-1)){
    tang[1] = Orte[nodex[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[1] = VectorSelected(nodex[ix+iy*size+iz*size2],nodey[ix+(iy-1)*size+iz*size2],
		    nodey[(ix+1)+(iy-1)*size+(iz)*size2],nodex[ix+(iy-1)*size+(iz)*size2]) ;
  }
  if(( iz == tief-1 )||(ix == breit-1)){
    tang[2] = Orte[nodex[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[2] = VectorSelected(nodex[ix+iy*size+iz*size2],nodez[ix+iy*size+iz*size2],
		    nodez[(1+ix)+(iy)*size+iz*size2],nodex[ix+iy*size+(iz+1)*size2]) ;
  }

  if(( iz == 0 )||(ix == breit-1)){
    tang[3] = Orte[nodex[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[3] = VectorSelected(nodex[ix+iy*size+iz*size2],nodez[ix+(iy)*size+(iz-1)*size2],
		    nodez[(ix+1)+(iy)*size+(iz-1)*size2],nodex[ix+(iy)*size+(iz-1)*size2]) ;
  }
  return (((tang[0] - tang[1])%(tang[2] - tang[3])) * grad[ix+iy*size+iz*size2]);
}
GLfloat Isosurf::CalcNormalY( int ix, int iy, int iz ) {
  Vector3 tang[4];

  if(( ix == breit-1 )||(iy == hoch-1)){
    tang[0] = Orte[nodey[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[0] =  VectorSelected(nodey[ix+iy*size+iz*size2],nodex[ix+iy*size+iz*size2],
		    nodex[ix+(iy+1)*size+iz*size2],nodey[1+ix+iy*size+iz*size2]);
  }

  if(( ix == 0 )||(iy == hoch-1)){
    tang[1] = Orte[nodey[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[1] =  VectorSelected(nodey[ix+iy*size+iz*size2],nodex[(ix-1)+(iy)*size+(iz)*size2],
		    nodex[(ix-1)+(iy+1)*size+(iz)*size2],nodey[(ix-1)+(iy)*size+(iz)*size2]) ;
  }

  if(( iz == tief-1 )||(iy == hoch-1)){
    tang[2] = Orte[nodey[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[2] =  VectorSelected(nodey[ix+iy*size+iz*size2],nodez[ix+iy*size+iz*size2],
		    nodez[ix+(iy+1)*size+iz*size2],nodey[ix+iy*size+(iz+1)*size2]) ;
  }

  if(( iz == 0 )||(iy == hoch-1)){
    tang[3] = Orte[nodey[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[3] =  VectorSelected(nodey[ix+iy*size+iz*size2],nodez[(ix)+(iy)*size+(iz-1)*size2],
		    nodez[(ix)+(iy+1)*size+(iz-1)*size2],nodey[(ix)+(iy)*size+(iz-1)*size2]) ;
  }

  return (((tang[2] - tang[3])%(tang[0] - tang[1]))*grad[ix+iy*size+iz*size2]);
}
GLfloat Isosurf::CalcNormalZ( int ix, int iy, int iz ) {
  Vector3 tang[4];

  if(( ix == breit-1 )||(iz == tief-1)){
    tang[0] = Orte[nodez[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[0] =  VectorSelected(nodez[ix+iy*size+iz*size2],nodex[ix+iy*size+iz*size2],
		    nodex[ix+iy*size+(iz+1)*size2],nodez[1+ix+iy*size+iz*size2]) ;
  }

  if(( ix == 0 )||(iz == tief-1)){
    tang[1] = Orte[nodez[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[1] =  VectorSelected(nodez[ix+iy*size+iz*size2],nodex[(ix-1)+(iy)*size+(iz)*size2],
		    nodex[(ix-1)+(iy)*size+(iz+1)*size2],nodez[(ix-1)+(iy)*size+(iz)*size2]) ;
  }

  if(( iy == hoch-1 )||(iz == tief-1)){
    tang[2] = Orte[nodez[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[2] =  VectorSelected(nodez[ix+iy*size+iz*size2],nodey[ix+iy*size+iz*size2],
		    nodey[ix+iy*size+(iz+1)*size2],nodez[ix+(iy+1)*size+iz*size2]) ;
  }

  if(( iy == 0 )||(iz == tief-1)){
    tang[3] = Orte[nodez[ix+iy*size+iz*size2].index].vertex;
  }else{
    tang[3] =  VectorSelected(nodez[ix+iy*size+iz*size2],nodey[(ix)+(iy-1)*size+(iz)*size2],
		    nodey[(ix)+(iy-1)*size+(iz+1)*size2],nodez[(ix)+(iy-1)*size+(iz)*size2]) ;
  }

  return (((tang[0] - tang[1])%(tang[2] - tang[3]))*grad[ix+iy*size+iz*size2]);
}
/*
Vector3 Isosurf::cN( int ix, int iy, int iz) {
  Vector3 n;
  n=Normalize(grad[ix-1+(iy-1)*size+(iz-1)*size2]);
  n*=-1.0;
  return n;
}
*/
void Isosurf::CalcNormals( void ) {
  //FILE *test= fopen("Directions.txt","a");
  //fprintf(test,"neu\n");
  for( int ix=0; ix<breit; ix++ ){
    for( int iy=0; iy<hoch; iy++ ){
      for( int iz=0; iz<tief; iz++ ){
	if( nodex[ix+iy*size+iz*size2] ){
	  Orte[nodex[ix+iy*size+iz*size2].index].direct = (CalcNormalX(ix,iy,iz)>0)?1:-1;
	  //fprintf(test,"%9d [%4d,%4d,%4d] D:%2d X\n",nodex[ix+iy*size+iz*size2].index,ix,iy,iz,Orte[nodex[ix+iy*size+iz*size2].index].direct);
	}
	if( nodey[ix+iy*size+iz*size2] ){
	  Orte[nodey[ix+iy*size+iz*size2].index].direct = (CalcNormalY(ix,iy,iz)>0)?1:-1;
	  //fprintf(test,"%9d [%4d,%4d,%4d] D:%2d Y\n",nodey[ix+iy*size+iz*size2].index,ix,iy,iz,Orte[nodey[ix+iy*size+iz*size2].index].direct);
	}
	if( nodez[ix+iy*size+iz*size2] ){
	  Orte[nodez[ix+iy*size+iz*size2].index].direct = (CalcNormalZ(ix,iy,iz)>0)?1:-1;
	  //fprintf(test,"%9d [%4d,%4d,%4d] D:%2d Z\n",nodez[ix+iy*size+iz*size2].index,ix,iy,iz,Orte[nodez[ix+iy*size+iz*size2].index].direct);
	}
      }
    }
  }
  //fclose(test);
}
void Isosurf::makeFaces(int nn, Node poly[] ){
  int i,n=0;
  int ly[13];
  for (int j=0;j<nn;j++)//zu nahe beieinander liegende verts ignorieren 
    if (Distance(Orte[poly[j].index].vertex,Orte[poly[(j+1)%nn].index].vertex)>0.00) 
      ly[n++]=poly[j].index;
    else{
      Orte[poly[j].index].vertex=
	      Orte[poly[(j+1)%nn].index].vertex=
	      (Orte[poly[j].index].vertex+Orte[poly[(j+1)%nn].index].vertex)*0.5f;
      Orte[poly[j].index].color=
	      Orte[poly[(j+1)%nn].index].color=
	      (Orte[poly[j].index].color+Orte[poly[(j+1)%nn].index].color)*0.5f;
      ly[n++]=poly[(j+1)%nn].index;
      j++;
    }
  GLfloat mid_col = 0.0;
  Vector3 mid_ver = Vector3(0,0,0);
  Vector3 mid_nor = Vector3(0,0,0);  
  for( i=0; i<n; i++ ){
    mid_ver += Orte[ly[i]].vertex;
    mid_nor += Orte[ly[i]].normal;
    mid_col += Orte[ly[i]].color;    
  }
  mid_ver *= (1.0/n);
  Normalize(mid_nor);
  mid_col *= (1.0/n);
  if (n<3) return;  //weniger als 3 verts -> nichts zu tun
  //printf("Vor Realoc %p\n",PGN);fflush(stdout);
  if (( PGN =(Polygn*)realloc(PGN,sizeof(Polygn)*(PCN+1)))==NULL) {
    fprintf(stderr ,"Less Memory (Polygons)!!\n");
    exit(1);  
  }

  //printf("Nach Realoc %p\n",PGN);fflush(stdout);
  VCN+=n;//vertszhler erhhen
  PGN[PCN].vertex=mid_ver;
  PGN[PCN].n=n;
  if (Orte[poly[0].index].direct*iso_level<0.0){//drehsinn der polygone abfragen*/
    for( i=0; i<n; i++ ){
      PGN[PCN].ii[i]=ly[i];
    }
  }else{
    int k=0;
    for( i=n-1; i>=0; i-- ){
      PGN[PCN].ii[i]=ly[k++];	
    }
  }//else*/  
  PCN++;//polygonzhler erhhen
  //if (f!=NULL) fprintf(f,"\n");
}
void Isosurf::DrawPlys(){
  glLoadName(4444);
  //if (!lastcall) return;
  for (int i=0;i<PCN;i++){
    switch (PGN[i].n){
	    case  3:glBegin(GL_TRIANGLES); 
		    for (int k=0;k<3;k++) {
		      //	    printf("#####%d %d %f\n",k,ly[k],C[ly[k]]);fflush(stdout);
		      Farbverlauf(Orte[PGN[i].ii[k]].color);
		      //printf("#####\n");fflush(stdout);
		      glTexCoord1f((Orte[PGN[i].ii[k]].color-min)/(max-min));
		      glNormal3fv( (GLfloat*)&Orte[PGN[i].ii[k]].normal);
		      glVertex3fv( (GLfloat*)&Orte[PGN[i].ii[k]].vertex);
		    }          
		    glEnd();
		    break;
	    case  4:if (Quad){
		      glBegin(GL_QUADS);
		      for (int k=0;k<4;k++) {
			Farbverlauf(Orte[PGN[i].ii[k]].color);
			glTexCoord1f((Orte[PGN[i].ii[k]].color-min)/(max-min));
			glNormal3fv( (GLfloat*)&Orte[PGN[i].ii[k]].normal);
			glVertex3fv( (GLfloat*)&Orte[PGN[i].ii[k]].vertex);
		      }
		      glEnd();
		      break;
		    }
	    case  5:
	    case  6:
	    case  7:
	    case  8:
	    case  9:
	    case  10:
	    case  11:
	    case   12:{
			Vector3 mO=Orte[PGN[i].ii[0]].vertex,mN=Orte[PGN[i].ii[0]].normal;
			GLfloat mC=Orte[PGN[i].ii[0]].color;
			for (int k=1;k <PGN[i].n; k++) {
			  mO+=Orte[PGN[i].ii[k]].vertex;
			  mN+=Orte[PGN[i].ii[k]].normal;
			  mC+=Orte[PGN[i].ii[k]].color;}
			  mC/=(GLfloat)PGN[i].n;
			  mO*=(GLfloat)1.0/PGN[i].n;
			  mN*=(GLfloat)1.0/PGN[i].n;         
			  glBegin(GL_TRIANGLE_FAN);
			  Farbverlauf(mC);
			  glTexCoord1f((mC-min)/(max-min));
			  glNormal3fv( (GLfloat*)&mN);
			  glVertex3fv( (GLfloat*)&mO);
		      }
		      for (int k=0;k<=PGN[i].n;k++) {Farbverlauf(Orte[PGN[i].ii[k%PGN[i].n]].color);
			glTexCoord1f((Orte[PGN[i].ii[k%PGN[i].n]].color-min)/(max-min));
			glNormal3fv( (GLfloat*)&Orte[PGN[i].ii[k%PGN[i].n]].normal);
			glVertex3fv( (GLfloat*)&Orte[PGN[i].ii[k%PGN[i].n]].vertex);
		      }
		      glEnd();
		      break;
	    default : printf("moehp!\n");glEnd();break;
    }
  }		
}	
/*void Isosurf::DrawPolygon( int nn, Node poly[] ){
  int i,n=0;
  glLoadName(6);
  int ly[13];
  for (int j=0;j<nn;j++) 
  if (Distance(Orte[poly[j].index].vertex,Orte[poly[(j+1)%nn].index].vertex)>0.00) 
  ly[n++]=poly[j].index;
  else{
  Orte[poly[j].index].vertex=
  Orte[poly[(j+1)%nn].index].vertex=
  (Orte[poly[j].index].vertex+Orte[poly[(j+1)%nn].index].vertex)*0.5f;
  Orte[poly[j].index].color=
  Orte[poly[(j+1)%nn].index].color=
  (Orte[poly[j].index].color+Orte[poly[(j+1)%nn].index].color)*0.5f;
  ly[n++]=poly[(j+1)%nn].index;
  j++;
  }
  GLfloat mid_col = 0.0;
  Vector3 mid_ver = Vector3(0,0,0);
  Vector3 mid_nor = Vector3(0,0,0);
  if (n<3) return;  
  VCN+=n;
  PCN++;
  for( i=0; i<n; i++ ){
  mid_ver += Orte[ly[i]].vertex;
  mid_nor += Orte[ly[i]].normal;
  mid_col += Orte[ly[i]].color;    
  }
  mid_ver *= (1.0/n);
  Normalize(mid_nor);
  mid_col *= (1.0/n);
  if (Orte[poly[0].index].direct<0){
  glColor4f(0.0,0.0,0.0,1.0);
  if( n==3 ){
  glBegin( GL_TRIANGLES );    
  for( i=0; i<n; i++ ){
  glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
  Farbverlauf(Orte[ly[i]].color);
  if (f!=NULL) fprintf(f,"%6d ",ly[i]);
  glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
  glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
  }
  glEnd();
  }else
  if(( n==4 )&&(Quad)){
  glBegin( GL_QUADS );    
  for( i=0; i<n; i++ ){
  glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
  Farbverlauf(Orte[ly[i]].color);
  if (f!=NULL) fprintf(f,"%6d ",ly[i]);  
  glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
  glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
  }
  glEnd();
  }else {     
  glBegin( GL_TRIANGLE_FAN );{
  glNormal3fv( (GLfloat*)&mid_nor );   	
  Farbverlauf(mid_col);
  Farbverlauf(mid_col);
  glTexCoord1f((mid_col-min)/(max-min));
  glVertex3fv( (GLfloat*)&mid_ver );
  for( i=0; i<n; i++ ){
  glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
  Farbverlauf(Orte[ly[i]].color);
  if (f!=NULL) fprintf(f,"%6d ",ly[i]);
  glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
  glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
  }
  i=0;{
  glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
  Farbverlauf(Orte[ly[i]].color);
glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
}
}glEnd();
}
}else{
  glColor4f(0.0,0.0,0.0,1.0);
  if( n==3 ){
    glBegin( GL_TRIANGLES );    
    for( i=n-1; i>=0; i-- ){
      glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
      Farbverlauf(Orte[ly[i]].color);
      if (f!=NULL) fprintf(f,"%6d ",ly[i]);
      glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
      glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
    }
    glEnd();
  }else 
    if(( n==4 )&&(Quad)){
      glBegin( GL_QUADS );    
      for( i=n-1; i>=0; i-- ){
	glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
	Farbverlauf(Orte[ly[i]].color);
	if (f!=NULL) fprintf(f,"%6d ",ly[i]);
	glTexCoord1f((Orte[ly[i]].color-min)/(max-min));	
	glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
      }
      glEnd();
    }else{
      glBegin( GL_TRIANGLE_FAN );{	
	glNormal3fv( (GLfloat*)&mid_nor );   
	Farbverlauf(mid_col);    
	glTexCoord1f((mid_col-min)/(max-min));
	glVertex3fv( (GLfloat*)&mid_ver );      
	for( i=n-1; i>=0; i-- ){	
	  glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
	  Farbverlauf(Orte[ly[i]].color);
	  if (f!=NULL) fprintf(f,"%6d ",ly[i]);
	  glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
	  glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
	}
	i=n-1;{
	  glNormal3fv( (GLfloat*)&Orte[ly[i]].normal );
	  Farbverlauf(Orte[ly[i]].color);
	  glTexCoord1f((Orte[ly[i]].color-min)/(max-min));
	  glVertex3fv( (GLfloat*)&Orte[ly[i]].vertex );
	}
      }glEnd();
    }
}

if (f!=NULL) fprintf(f,"\n");
}*/
void Isosurf::Farbverlauf (GLfloat wrt){
  static GLclampd ff[4];
  static GLdouble anf,lang;
  static int Farben=6;
  static int lauf=0;
  int keinPG;
  static GLclampd farbe[7][4]={{0.6,0,0,0.5},
    {0.6,0.5,0,0.5},
    {0,0.5,0,0.5},
    {0,0.5,0.9,0.5},
    {0,0,0.9,0.5},
    {0.6,0,0.9,0.5},
    {0.6,0,0,0.5}};
  keinPG=1;
  //  printf("DAOKf%d\n",erstma);
  if (erstma) {
    FILE *FFL;
    int cnt=0;
    Farben=6;
    //printf("DAOKf%d\n",plana);

    if ((plana)&&((FFL=fopen("PGradient.dat","r"))!=NULL)) {
      printf("Reading PGradient.dat\n");
      keinPG=0;
      while ((cnt<7)&&(!feof(FFL))) {fscanf(FFL,"%lf %lf %lf %lf\n",&farbe[cnt][0],&farbe[cnt][1],&farbe[cnt][2],&farbe[cnt][3]);
	printf("%f %f %f %f\n",farbe[cnt][0],farbe[cnt][1],farbe[cnt][2],farbe[cnt][3]);
	cnt++;}

	fclose(FFL);
	Farben=cnt;
	printf("%d Color gradient colors.\n",Farben);
    }
    if (((!plana)||(keinPG))&&((FFL=fopen("Gradient.dat","r"))!=NULL)) {
      printf("Reading Gradient.dat\n");
      while ((cnt<7)&&(!feof(FFL))) {fscanf(FFL,"%lf %lf %lf %lf\n",&farbe[cnt][0],&farbe[cnt][1],&farbe[cnt][2],&farbe[cnt][3]);
	printf("%f %f %f %f\n",farbe[cnt][0],farbe[cnt][1],farbe[cnt][2],farbe[cnt][3]);
	cnt++;}
	fclose(FFL);
	Farben=cnt;
	printf("%d Color gradient colors.\n",Farben);
    }   

    //printf("DAOK mmm\n");  
    erstma=0;
  }
  wrt=(wrt-min)/(max-min);

  wrt=(wrt>=1.0)?0.999999:wrt;
  wrt=(wrt<=0.0)?0.000001:wrt;
  /*  const GLdouble K=4.0;
      Vector3 b;
#define max(x,y) ((x>y)?x:y)
b.x=max(0.6-K*(wrt-0.89)*(wrt-0.89),0.0)+max(0.7-K*(wrt+0.15)*(wrt+0.15),0.0);
b.y=0.6-K*(wrt-0.65)*(wrt-0.65);
b.z=0.9-K*(wrt-0.20)*(wrt-0.20);
ff[0]=(b.x>0)?b.x:0.0;
ff[1]=(b.y>0)?b.y:0.0;
ff[2]=(b.z>0)?b.z:0.0;
ff[3]=0.5;

*/

  lang=1.0/(Farben-1.0);
  lauf=(int (wrt/lang));

  anf=lang*lauf;

  //  printf("%d %f %f %f\n",lauf,lang,anf,wrt);  
  ff[0]=((1.0-(wrt-anf)/lang)*farbe[lauf][0]+((wrt-anf)/lang)*farbe[lauf+1][0]);
  ff[1]=((1.0-(wrt-anf)/lang)*farbe[lauf][1]+((wrt-anf)/lang)*farbe[lauf+1][1]);
  ff[2]=((1.0-(wrt-anf)/lang)*farbe[lauf][2]+((wrt-anf)/lang)*farbe[lauf+1][2]);
  ff[3]=(1.0-(wrt-anf)/lang)*farbe[lauf][3]+((wrt-anf)/lang)*farbe[lauf+1][3];  

  glColor4dv(ff);

  //printf("DAOKf2\n");
  return;
}
void Isosurf::minmax(void){//Mini-& Maximalwert der Isooberflache bestimmen.
  FILE* in;
  // printf("plana:%d % min%f max%f\n",plana,min,max);
  if ((plana)&&((in=fopen("Pminmax.in","r"))!=NULL)) {
    fscanf(in,"%f %f\n",&min,&max);
    fclose(in);
  } else
    if ((!plana)&&((in=fopen("minmax.in","r"))!=NULL)) {
      fscanf(in,"%f %f\n",&min,&max);
      fclose(in);
    }
  //  double avpot,avpotminus,avpotplus,avdevPI,potsigtot,potsigminus,potsigplus,potnu;
  /*potlizer et al Int. J. Quant. Chem. 85 676-684*/
  //int potN,potalp,potbet; 
    else
    {fprintf(stderr,"can't open %cminmax.in for reading. Now calculating min and max value of the %s\n",(plana)?'P':' ',(plana)?"CutPlane":"isosurface");        
      double avpot=0.0,avpotminus=0.0,avpotplus=0.0,avdevPI=0.0,potsigtot=0.0,potsigminus=0.0,potsigplus=0.0,potnu=0.0;
      /*potlizer et al Int. J. Quant. Chem. 85 676-684*/
      int potalp=0,potbet=0;
      int minI=0;
      int maxI=0;	
      for (int i=0;i<ozi;i++) {
	if(!plana) {
	  minI=(Orte[i].color<min)?i:minI;
	  maxI=(Orte[i].color>max)?i:maxI;
	}
	min=(Orte[i].color<min)?Orte[i].color:min;
	max=(Orte[i].color>max)?Orte[i].color:max;
	if(!plana) {
	  avpot+=Orte[i].color*lucf;
	  if (Orte[i].color> 0) {potalp++;avpotplus+=Orte[i].color*lucf;}
	  if (Orte[i].color< 0) {potbet++;avpotminus+=Orte[i].color*lucf;}}
      }
      if(!plana) {
	printf("\ninternal moliso coordinates of VS,min and VS,max\n%f %s @ %8.6f %8.6f %8.6f\n %f %s @ %8.6f %8.6f %8.6f\n",		
			Orte[minI].color*lucf,lul,
			Orte[minI].vertex.x,
			Orte[minI].vertex.y,
			Orte[minI].vertex.z,
			Orte[maxI].color*lucf,lul,
			Orte[maxI].vertex.x,
			Orte[maxI].vertex.y,
			Orte[maxI].vertex.z);
	printf("xd-grid-file coordinates of VS,min and VS,max\n%f %s @ %7.5f %7.5f %7.5f\n %f %s @ %7.5f %7.5f %7.5f\n",		
			Orte[minI].color*lucf,lul,
			(Orte[minI].vertex.x-0.5)*dim ,
			(Orte[minI].vertex.y-0.5)*dim , 
			(Orte[minI].vertex.z-0.5)*dim ,
			Orte[maxI].color*lucf,lul,
			(Orte[maxI].vertex.x-0.5)*dim ,
			(Orte[maxI].vertex.y-0.5)*dim ,
			(Orte[maxI].vertex.z-0.5)*dim) ;
	//1.624980   -5.330702   -0.042628
	//printf("%f %f %f \n",(((0.405871 -0.5)*(-dim))+GA)/0.529177249,
	//  (((0.868003 -0.5)*(-dim))+GB)/0.529177249,
	//  (((0.537344 -0.5)*(-dim))+GC)/0.529177249);

	printf("cube-file coordinates of VS,min and VS,max\n%3.5E %s @ %7.5f %7.5f %7.5f\n %3.5E %s @ %7.5f %7.5f %7.5f \n",		
			Orte[minI].color*lucf,lul,
			(((Orte[minI].vertex.x-0.5)*(dim))+GA)/0.529177249 ,
			(((Orte[minI].vertex.y-0.5)*(dim/yrat))+GB)/0.529177249 ,
			(((Orte[minI].vertex.z-0.5)*(dim/zrat))+GC)/0.529177249 ,
			Orte[maxI].color*lucf,lul,
			(((Orte[maxI].vertex.x-0.5)*(dim))+GA)/0.529177249 ,
			(((Orte[maxI].vertex.y-0.5)*(dim/yrat))+GB)/0.529177249 ,
			(((Orte[maxI].vertex.z-0.5)*(dim/zrat))+GC)/0.529177249) ;

	avpot/=(double)ozi;
	avpotplus/=(double)potalp;
	avpotminus/=(double)potbet;
	for (int i=0;i<ozi;i++) {
	  avdevPI+=fabs(Orte[i].color*lucf-avpot);
	  if (Orte[i].color> 0) {potsigplus+=pow(Orte[i].color*lucf-avpot,2.0);}
	  if (Orte[i].color< 0) {potsigminus+=pow(Orte[i].color*lucf-avpot,2.0);}
	}
	potsigplus/=(double)potalp;
	potsigminus/=(double)potbet;
	potsigtot=potsigplus+potsigminus;
	avdevPI/=(double)ozi;
	potnu=potsigplus*potsigminus/(potsigtot*potsigtot);
	printf("\nAvarage of positive surface values VS+= %f %s\n Avarage of negative surface values VS-= %f %s\n Avarage deviation from the avarage surface value PI= %f %s\n sigma square + = %f (%s)^2\n sigma square - = %f (%s)^2\n sigma square total = %f (%s)^2\n nu = %f \n\n",
			avpotplus,lul,avpotminus,lul,avdevPI,lul,potsigplus,lul,potsigminus,lul,potsigtot,lul,potnu);
	printf("Number of positive grid-points %d\nNumber of negative grid-points %d\nTotal number %d and Sum %d \n\n",potalp,potbet,ozi,potalp+potbet);
      }
    }
  max=max+0.0001;
  min=min-0.0001;
  FILE* out;
  if (plana) if ((out=fopen("Pminmax.out","w"))==NULL){fprintf(stderr,"can't open Pminmax.out for writing\n");}
  else{
    fprintf(out,"%16.12f %16.12f\n",min,max);
    fclose(out);
  } else
    if ((out=fopen("minmax.out","w"))==NULL){fprintf(stderr,"can't open minmax.out for writing\n");}
    else{
      fprintf(out,"%16.12f %16.12f\n",min,max);
      fclose(out);
    }
}
void Isosurf::MakeElement( int ix, int iy, int iz ,int s1, int s2) {//das ist der Teil des japanischen Programms den ich nicht verstehe.
  //Hauptsache fuktioniert.
  static int conn[12][2][4] = {           //char->int wegen warning g++ >3.0
    {{ 0, 1, 7, 6}, { 0, 2, 8, 3}},  //  0
    {{ 1, 2, 5, 4}, { 1, 0, 6, 7}},  //  1
    {{ 2, 0, 3, 8}, { 2, 1, 4, 5}},  //  2
    {{ 3, 8, 2, 0}, { 3, 4,10, 9}},  //  3
    {{ 4, 3, 9,10}, { 4, 5, 2, 1}},  //  4
    {{ 5, 4, 1, 2}, { 5, 6, 9,11}},  //  5
    {{ 6, 5,11, 9}, { 6, 7, 1, 0}},  //  6
    {{ 7, 6, 0, 1}, { 7, 8,11,10}},  //  7
    {{ 8, 7,10,11}, { 8, 3, 0, 2}},  //  8
    {{ 9,10, 4, 3}, { 9,11, 5, 6}},  //  9
    {{10,11, 8, 7}, {10, 9, 3, 4}},  // 10
    {{11, 9, 6, 5}, {11,10, 7, 8}}   // 11
  };
  static Node node[12];
  static Node polygon[12];

  node[ 0] = nodex[ix+iy*s1+iz*s2];        // 000x
  node[ 1] = nodey[ix+iy*s1+iz*s2];        // 000y
  node[ 2] = nodez[ix+iy*s1+iz*s2];        // 000z
  node[ 3] = nodex[ix+iy*s1+(iz+1)*s2];    // 001y 
  node[ 4] = nodey[ix+iy*s1+(iz+1)*s2];    // 001z 
  node[ 5] = nodez[ix+(iy+1)*s1+iz*s2];    // 010x 
  node[ 6] = nodex[ix+(iy+1)*s1+iz*s2];    // 010y 
  node[ 7] = nodey[1+ix+iy*s1+iz*s2];      // 100y
  node[ 8] = nodez[1+ix+iy*s1+iz*s2];      // 100z
  node[ 9] = nodex[ix+(iy+1)*s1+(iz+1)*s2];// 011x
  node[10] = nodey[(ix+1)+iy*s1+(iz+1)*s2];// 101y
  node[11] = nodez[(ix+1)+(iy+1)*s1+iz*s2];// 110z
  if (((char)node[0]+node[1]+node[2]+node[3]+node[4]+node[5]
			  +node[6]+node[7]+node[8]+node[9]+node[10]+node[11])==0) return;
  for( int is=0; is<12; is++ ) {
    if( !node[is] ) continue;

    int n=0, i=is, m=0,ai=i;
    do {
      polygon[n++]= node[i];
      int sol = IndexSelected(node[conn[i][m][0]],node[conn[i][m][1]],
		      node[conn[i][m][2]],node[conn[i][m][3]]);
      ai=i;
      i = conn[i][m][sol];

      if( sol == 2 ) m ^= 1;
      node[i].flag = 0;
    }while( (i!=is)&&(n<11) );    
    {
      //DrawPolygon( n, polygon );
      makeFaces( n, polygon );
    }
  }    
}
void Isosurf::Regularize(void){  
  ort *ooo;
  printf("Regularizing polygons.\n");
  ooo=(ort*)malloc((ozi+1)*sizeof(ort));
  //FILE *f=fopen("Regel.txt","w");
  Vector3 sumV,sumN,NVx;
  int s1=breit,s2=breit*hoch,nb=0;
  int ux,uy,uz,ox,oy,oz;
  GLfloat sumC,RW;
  int iDx[50];
  for (int ix=0;ix<breit;ix++)    
    for (int iy=0;iy<hoch;iy++)    
      for (int iz=0;iz<tief;iz++){//xyz
	ox=(ix<breit-1)?2:1;
	oy=(iy<hoch-1)?2:1;
	oz=(iz<tief-1)?2:1;
	ux=(ix>0)?-1:0;
	uy=(iy>0)?-1:0;
	uz=(iz>0)?-1:0;
	if (nodex[ix+iy*s1+iz*s2]){
	  nb=1;
	  RW=0; 
	  iDx[nb-1]=nodex[ix+iy*s1+iz*s2].index;
	  sumV=Orte[nodex[ix+iy*s1+iz*s2].index].vertex;
	  sumN=Orte[nodex[ix+iy*s1+iz*s2].index].normal;
	  sumC=Orte[nodex[ix+iy*s1+iz*s2].index].color;
	  for (int i=ux;i<ox;i++)
	    for (int j=uy;j<oy;j++)
	      for (int k=uz;k<oz;k++) {
		if (nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {
		  nb++;      
		  iDx[nb-1]=nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex;
		  sumN+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal;
		  sumC+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}
		if (nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {                
		  nb++;    
		  iDx[nb-1]=nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex; 
		  sumN+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal; 
		  sumC+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}            
		if (nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {              
		  nb++;   
		  iDx[nb-1]=nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex;
		  sumN+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal;
		  sumC+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}            	    
	      }
	  NVx=Normalize(sumN);
	  for (int kk=0;kk<nb;kk++) {
	    RW+=pow(Orte[iDx[kk]].normal*NVx,2.0f);}
	  RW/=(float)nb;
	  RW=1.0-RW;
	  if (nb){
	    ooo[nodex[ix+iy*s1+iz*s2].index].vertex=RW*Orte[nodex[ix+iy*s1+iz*s2].index].vertex+ (1.0-RW)*sumV*(1.0/nb);
	    ooo[nodex[ix+iy*s1+iz*s2].index].normal=Normalize(sumN);
	    ooo[nodex[ix+iy*s1+iz*s2].index].color=RW*Orte[nodex[ix+iy*s1+iz*s2].index].color+   (1.0-RW)*sumC/nb;
	  }
	}
	if (nodey[ix+iy*s1+iz*s2]){//y
	  nb=1;
	  RW=0; 
	  iDx[nb-1]=nodey[ix+iy*s1+iz*s2].index;
	  sumV=Orte[nodey[(ix)+(iy)*s1+(iz)*s2].index].vertex;
	  sumN=Orte[nodey[(ix)+(iy)*s1+(iz)*s2].index].normal;
	  sumC=Orte[nodey[(ix)+(iy)*s1+(iz)*s2].index].color;
	  for (int i=ux;i<ox;i++)
	    for (int j=uy;j<oy;j++)
	      for (int k=uz;k<oz;k++) {
		if (nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {
		  nb++;    
		  iDx[nb-1]=nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex;
		  sumN+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal;
		  sumC+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}
		if (nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {                
		  nb++;     
		  iDx[nb-1]=nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex; 
		  sumN+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal; 
		  sumC+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}            
		if (nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {              
		  nb++;    
		  iDx[nb-1]=nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex;
		  sumN+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal;
		  sumC+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}            	    
	      }

	  NVx=Normalize(sumN);
	  for (int kk=0;kk<nb;kk++) {
	    RW+=pow(Orte[iDx[kk]].normal*NVx,2.0f);}
	  RW/=(float)nb;
	  RW=1.0-RW;
	  if (nb){
	    ooo[nodey[ix+iy*s1+iz*s2].index].vertex=RW*Orte[nodey[ix+iy*s1+iz*s2].index].vertex+ (1.0-RW)*sumV*(1.0/nb);
	    ooo[nodey[ix+iy*s1+iz*s2].index].normal=Normalize(sumN);
	    ooo[nodey[ix+iy*s1+iz*s2].index].color=RW*Orte[nodey[ix+iy*s1+iz*s2].index].color+(1.0-RW)*sumC/nb;
	  }
	}
	if (nodez[ix+iy*s1+iz*s2]){	
	  nb=1;
	  RW=0; 
	  iDx[nb-1]=nodez[ix+iy*s1+iz*s2].index;
	  sumV=Orte[nodez[(ix)+(iy)*s1+(iz)*s2].index].vertex;
	  sumN=Orte[nodez[(ix)+(iy)*s1+(iz)*s2].index].normal;
	  sumC=Orte[nodez[(ix)+(iy)*s1+(iz)*s2].index].color;
	  for (int i=ux;i<ox;i++)
	    for (int j=uy;j<oy;j++)
	      for (int k=uz;k<oz;k++) {
		if (nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {
		  nb++;    
		  iDx[nb-1]=nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex;
		  sumN+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal;
		  sumC+=Orte[nodex[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}
		if (nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {                
		  nb++;     
		  iDx[nb-1]=nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex; 
		  sumN+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal; 
		  sumC+=Orte[nodey[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color; 
		}            
		if (nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2]) {              
		  nb++;    
		  iDx[nb-1]=nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index;
		  sumV+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].vertex;
		  sumN+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].normal;
		  sumC+=Orte[nodez[(ix+i)+(iy+j)*s1+(iz+k)*s2].index].color;
		}            	    
	      }
	  NVx=Normalize(sumN);
	  for (int kk=0;kk<nb;kk++) {
	    RW+=pow(Orte[iDx[kk]].normal*NVx,2.0f);}
	  RW/=(float)nb;
	  RW=1.0-RW;
	  if (nb){
	    ooo[nodez[ix+iy*s1+iz*s2].index].vertex=RW*Orte[nodez[ix+iy*s1+iz*s2].index].vertex+ (1.0-RW)*sumV*(1.0/nb);
	    ooo[nodez[ix+iy*s1+iz*s2].index].normal=Normalize(sumN);
	    ooo[nodez[ix+iy*s1+iz*s2].index].color=RW*Orte[nodez[ix+iy*s1+iz*s2].index].color+(1.0-RW)*sumC/nb;
	  }
	}
      }    
  for (int i=pcn_start;i<ozi;i++){
    if ((fabs(Norm(ooo[i].vertex))<1.2)&&(fabs(Norm(ooo[i].vertex))>0.9)){Orte[i].vertex=ooo[i].vertex;
      Orte[i].normal=ooo[i].normal;
      Orte[i].color=ooo[i].color;}
  }	
  free(ooo);
  CalcNormals();
}
void Isosurf::Free(void){
  if (grad!=NULL) free(grad);grad=NULL;
  if (nodex!=NULL) free(nodex);nodex=NULL;
  if (nodey!=NULL) free(nodez);nodey=NULL;
  if (nodez!=NULL) free(nodey);nodez=NULL;
}
void Isosurf::save(void) {	
  for (int i=0; i<PCN;i++){		
    for (int j=0; j<PGN[i].n;j++)			
      fprintf(f,"%6d ",PGN[i].ii[j]);		
    if (PGN[i].n>0) fprintf(f,"\n");	
  }
}
void Isosurf::MakeElements( void ) {  
  for( int ix=0; ix<breit-1; ix++ )
    for( int iy=0; iy<hoch-1; iy++ )      
      for( int iz=0; iz<tief-1; iz++ )	
	MakeElement(ix,iy,iz,breit,bh); ;
  PXsort();
  if (f!=NULL) save();
  DrawPlys();

}
void Isosurf::MakeElements2( void ) {  
  /*for( int ix=breit-2; ix>-1; ix-- ){
    for( int iy=hoch-2; iy>-1; iy-- ){
    for( int iz=tief-2; iz>-1; iz-- ){	
    MakeElement(ix,iy,iz,breit,bh);
    }
    }
    }*/
  Pxsort();
  //if (f!=NULL) save();
  DrawPlys();
}
void Isosurf::MakeElements3( void ) { 
  PYsort();
  //if (f!=NULL) save();
  DrawPlys();
  /*for( int iy=0; iy<hoch-1; iy++ ){
    for( int ix=0; ix<breit-1; ix++ ){      
    for( int iz=0; iz<tief-1; iz++ ){	
    MakeElement(ix,iy,iz,breit,bh);
    }
    }
    }*/
}
void Isosurf::MakeElements4( void ) {  
  /*for( int iy=hoch-2; iy>-1; iy-- ){
    for( int ix=breit-2; ix>-1; ix-- ){
    for( int iz=tief-2; iz>-1; iz-- ){	
    MakeElement(ix,iy,iz,breit,bh);
    }
    }
    }*/
  Pysort();
  //if (f!=NULL) save();
  DrawPlys();
}
void Isosurf::MakeElements5( void ) {  
  /*for( int iz=0; iz<tief-1; iz++ ){
    for( int ix=0; ix<breit-1; ix++ ){      
    for( int iy=0; iy<hoch-1; iy++ ){	
    MakeElement(ix,iy,iz,breit,bh);
    }
    }
    }*/
  PZsort();
  //if (f!=NULL) save();
  DrawPlys();
}
void Isosurf::MakeElements6( void ) {  
  /*for( int iz=tief-2; iz>-1; iz-- ){
    for( int ix=breit-2; ix>-1; ix-- ){      
    for( int iy=hoch-2; iy>-1; iy-- ){	
    MakeElement(ix,iy,iz,breit,bh);
    }
    }
    } */
  Pzsort();
  //if (f!=NULL) save();
  DrawPlys();
}
void Isosurf::Legende( void ) {
  char zz[80]="";
  GLfloat fw=0.0;  
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glOrtho( 0,1,0,1,-1,1);
  glMatrixMode(GL_MODELVIEW );
  glLoadIdentity();   
  glPushMatrix();
  glBegin( GL_TRIANGLE_STRIP );
  for (int j=0;j<256;j++){		
    Farbverlauf((0.0039*j*(max-min))+min);
    glTexCoord1f(0.0039*j);
    glVertex3f(0.05,0.55+.00156f*(float)j,0.0f);
    glVertex3f(0.17,0.55+.00156f*(float)j,0.0f);
  }
  glEnd();
  glDisable(GL_TEXTURE_1D);
  for (int i=0;i<6;i++) {
    fw=(0.2*i*(max-min))+min;
    Farbverlauf(fw); 
    glRasterPos3f(0.2,0.55+0.08f*(float)i,0.0f);	
    if (i!=5) sprintf(zz,"%5.3f",fw*lucf);
    else sprintf(zz,"%5.3f %s",fw*lucf,lul);
    glPrint(zz);}  
    glPopMatrix();  
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 29.0, (GLdouble)_win_width/_win_height, 50.0, 800.0 );
    glMatrixMode(GL_MODELVIEW); 

}
